@@ -10,12 +10,17 @@
 #' @param selected_sender Specific sender cell type to filter, default is None (use all) (character).
 #' @param selected_receiver Specific receiver cell type to filter, default is None (use all) (character).
 #' @param selected_metadata List of column names in \code{metadata} to annotate samples (default: None, use all)(character vector).
+#' @param treeheight_row The height of a tree for rows (numeric, default: 50).
+#' @param treeheight_col The height of a tree for columns (numeric, default: 50).
+#' @param show_LR Whether to display ligand-receptor names on rows (logical, default: FALSE).
+#' @param show_sample Whether to display sample names on columns (logical, default: FALSE).
+#' @param basic_title Custom heatmap title (optional).
 #'
-#' @return Heatmap of average LR interaction scores per sample.
+#' @return A pheatmap object.
 #'
 #' @export
 #'
-#' @importFrom dplyr %>% filter mutate group_by summarise
+#' @importFrom dplyr %>% filter mutate group_by summarise distinct
 #' @importFrom tidyr pivot_wider
 #' @importFrom pheatmap pheatmap
 #' @importFrom RColorBrewer brewer.pal
@@ -28,44 +33,61 @@
 #' # Heatmap of LR Interaction Scores
 #' data(lr_scores_eg)
 #' data(metadata_eg)
-#' p <- heatmap_sample(lr_scores_eg, metadata_eg, score = "normalized", selected_sender = "Cardiac",
-#'   selected_receiver = "Perivascular", selected_metadata = c("Sex", "Age_group", "IFN_type"))
+#'
+#' p <- heatmap_sample(
+#'   lr_scores = lr_scores_eg,
+#'   metadata = metadata_eg,
+#'   score = "normalized",
+#'   selected_sender = "Endothelial",
+#'   selected_receiver = "Perivascular",
+#'   selected_metadata = c("Sex", "Age_group", "IFN_type")
+#'   )
+#'
 #' print(p)
 heatmap_sample <- function(lr_scores,
                            metadata,
                            score = c("normalized", "raw"),
                            selected_sender = NULL,
                            selected_receiver = NULL,
-                           selected_metadata = NULL) {
+                           selected_metadata = NULL,
+                           treeheight_row = 50,
+                           treeheight_col = 50,
+                           show_LR = FALSE,
+                           show_sample = FALSE,
+                           basic_title = NULL) {
 
   # Parameter validation
   score <- match.arg(score)
   score_col <- ifelse(score == "normalized", "normalized_score", "score")
 
   if (!is.null(selected_sender)) {
-    lr_scores <- lr_scores %>% filter(.data[["sender"]] == selected_sender)
+    lr_scores <- lr_scores %>% filter(.data[["sender"]] %in% selected_sender)
   }
   if (!is.null(selected_receiver)) {
-    lr_scores <- lr_scores %>% filter(.data[["receiver"]] == selected_receiver)
+    lr_scores <- lr_scores %>% filter(.data[["receiver"]] %in% selected_receiver)
   }
 
-  # Create a new identifier by concatenating ligand, receptor, sender, and receiver
+  # Create LRSR key
   lr_scores <- lr_scores %>%
-    mutate(LRSR = paste(.data[["ligand"]], .data[["receptor"]], .data[["sender"]], .data[["receiver"]], sep = "_"))
+    mutate(
+      LRSR = paste(.data[["ligand"]], .data[["receptor"]], .data[["sender"]], .data[["receiver"]], sep = "_"),
+      LR = paste(.data[["ligand"]], .data[["receptor"]], sep = "_")
+    )
 
-  # Pivot data: calculate the average score for each LRSR-sample pair
+  # Create heatmap matrix
   heatmap_data <- lr_scores %>%
     group_by(.data[["LRSR"]], sample) %>%
     summarise(mean_score = mean(.data[[score_col]], na.rm = TRUE), .groups = "drop") %>%
     pivot_wider(names_from = sample, values_from = mean_score, values_fill = list(mean_score = NA))
     # pivot_wider(names_from = sample, values_from = mean_score, values_fill = list(mean_score = 0))
 
-  # Convert to matrix with LRSR as row names
   heatmap_matrix <- as.data.frame(heatmap_data)
   rownames(heatmap_matrix) <- heatmap_matrix$LRSR
   heatmap_matrix <- heatmap_matrix[,-1]
 
-  # Prepare annotation for columns using metadata.
+  display_names <- lr_scores$LR[match(rownames(heatmap_matrix), lr_scores$LRSR)]
+
+  # Prepare column annotation
   annotation_col <- NULL
 
   if ("sample" %in% colnames(metadata)) {
@@ -104,30 +126,96 @@ heatmap_sample <- function(lr_scores,
   annotation_col[] <- lapply(annotation_col, as.factor)
 
   # set color
-  auto_annotation_colors <- list()
+  get_base_colors <- function(n) {
+    if (n < 3) {
+      return(brewer.pal(3, "Paired")[1:n])
+    } else {
+      return(brewer.pal(min(9, n), "Paired"))
+    }
+  }
+
+  #  Set color for col annotations
+  auto_annotation_col_colors <- list()
   for(colname in colnames(annotation_col)){
     levels_current <- levels(annotation_col[[colname]])
     n_levels <- length(levels_current)
-    if(n_levels < 3){
-      base_colors <- brewer.pal(3, "Paired")[1:n_levels]
-    } else {
-      base_colors <- brewer.pal(min(9, n_levels), "Paired")
-    }
+    base_colors <- get_base_colors(n_levels)
     auto_colors <- colorRampPalette(base_colors)(n_levels)
-    auto_annotation_colors[[colname]] <- setNames(auto_colors, levels_current)
+    auto_annotation_col_colors[[colname]] <- setNames(auto_colors, levels_current)
   }
 
+  # Construct row annotations with sender and receiver
+  lr_annotations <- lr_scores %>%
+    distinct(.data[["LRSR"]], .data[["sender"]], .data[["receiver"]]) %>%
+    column_to_rownames("LRSR")
+
+  lr_annotations <- lr_annotations[rownames(heatmap_matrix), , drop = FALSE]
+  lr_annotations[] <- lapply(lr_annotations, as.factor)
+
+  # Rename columns to capitalize first letter
+  colnames(lr_annotations) <- c("Sender", "Receiver")
+
+  # Set color for row annotations
+  celltypes <- unique(c(
+    levels(lr_annotations$Sender),
+    levels(lr_annotations$Receiver)
+  ))
+  n_celltypes <- length(celltypes)
+
+  base_colors <- get_base_colors(n_celltypes)
+  unified_colors <- colorRampPalette(base_colors)(n_celltypes)
+  celltype_color_map <- setNames(unified_colors, celltypes)
+
+  auto_annotation_row_colors <- list(
+    Receiver = celltype_color_map[levels(lr_annotations$Receiver)],
+    Sender = celltype_color_map[levels(lr_annotations$Sender)]
+  )
+
+  lr_annotations <- lr_annotations[, c("Receiver", "Sender"), drop = FALSE]
+
+  # title
+  n_samples <- ncol(heatmap_matrix)
+  n_LRSR <- nrow(heatmap_matrix)
+
+  n_samples_fmt <- format(n_samples, big.mark = ",", scientific = FALSE)
+  n_LRSR_fmt <- format(n_LRSR, big.mark = ",", scientific = FALSE)
+
+  if (is.null(selected_sender) && is.null(selected_receiver)) {
+    tag <- "AllCells"
+  } else {
+    tag <- paste0(
+      if (!is.null(selected_sender)) paste(selected_sender, collapse = "+") else "All",
+      "_to_",
+      if (!is.null(selected_receiver)) paste(selected_receiver, collapse = "+") else "All"
+    )
+  }
+
+  # basic_title <- ifelse(tag == "AllCells", "All Cell-Cell Communication", gsub("_to_", "->", tag))
+  if (is.null(basic_title) || basic_title == "") {
+    basic_title <- if (tag == "AllCells") {
+      "All Cell-Cell Communication"
+    } else {
+      gsub("_to_", "->", tag)
+    }
+  }
+  heatmap_title <- sprintf("%s\nSamples: %s, LRSR: %s", basic_title, n_samples_fmt, n_LRSR_fmt)
+
   # Plot the heatmap using pheatmap
   ph <- pheatmap(heatmap_matrix,
                  color = colorRampPalette(rev(brewer.pal(n = 7, name = "RdYlBu")))(100),
+                 annotation_row = lr_annotations,
                  annotation_col = annotation_col,
-                 annotation_colors = auto_annotation_colors,
+                 annotation_colors = c(auto_annotation_row_colors, auto_annotation_col_colors),
+                 treeheight_row	= treeheight_row,
+                 treeheight_col = treeheight_col,
                  cluster_rows = TRUE,
                  cluster_cols = TRUE,
-                 show_rownames = FALSE,
-                 show_colnames = FALSE,
-                 fontsize = 8,
-                 border_color = "gray")
+                 show_rownames = show_LR,
+                 show_colnames = show_sample,
+                 labels_row = display_names,
+                 fontsize = 7,
+                 border_color = "gray",
+                 main = heatmap_title)
 
   return(ph)
 }
@@ -147,7 +235,11 @@ heatmap_sample <- function(lr_scores,
 #' @param color_by \code{metadata} column name to color points in PCA plot (character).
 #' @param n_components Number of principal components to extract (numeric, default: 2).
 #'
-#' @return A list with two elements: the first is a ggplot2 PCA scatter plot and the second is the PCA results data frame.
+#' @return A list containing:
+#' \itemize{
+#'   \item plot - ggplot object of the PCA scatter plot
+#'   \item df - data frame used for the PCA results
+#' }
 #'
 #' @export
 #'
@@ -161,8 +253,12 @@ heatmap_sample <- function(lr_scores,
 #' # PCA of LR Interaction Scores
 #' data(lr_scores_eg)
 #' data(metadata_eg)
-#' res <- pca_sample(lr_scores_eg, metadata_eg, selected_sender = "Cardiac",
-#'   selected_receiver = "Perivascular", color_by = "IFN_type")
+#'
+#' res <- pca_sample(
+#'   lr_scores = lr_scores_eg,
+#'   metadata = metadata_eg,
+#'   color_by = "IFN_type"
+#'   )
 #'
 #' print(res$plot)
 #' head(res$df)
@@ -235,15 +331,14 @@ pca_sample <- function(lr_scores,
       geom_point(size = 3) +
       labs(x = "Principal Component 1", y = "Principal Component 2",
            title = "PCA of Ligand-Receptor Interaction Scores", color = color_by) +
-      theme_minimal() +
+      theme_bw() +
       theme(
-        axis.text.x = element_text(color = "black"),
-        axis.text.y = element_text(color = "black"),
+        axis.text = element_text(color = "black"),
         plot.title = element_text(size = 12, hjust = 0.5, color = "black"),
         legend.position = "right",
-        axis.line = element_line(),
-        axis.ticks = element_line(),
-        panel.border = element_rect(color = "black", fill = NA),
+        # axis.line = element_line(),
+        # axis.ticks = element_line(),
+        # panel.border = element_rect(color = "black", fill = NA),
         panel.grid.major = element_blank(),
         panel.grid.minor = element_blank()
       ) +
@@ -255,20 +350,20 @@ pca_sample <- function(lr_scores,
       geom_point(size = 3, color = "black") +
       labs(x = "Principal Component 1", y = "Principal Component 2",
            title = "PCA of Ligand-Receptor Interaction Scores") +
-      theme_minimal() +
+      theme_bw() +
       theme(
-        axis.text.x = element_text(color = "black"),
-        axis.text.y = element_text(color = "black"),
+        axis.text = element_text(color = "black"),
         plot.title = element_text(size = 12, hjust = 0.5, color = "black"),
-        axis.line = element_line(),
-        axis.ticks = element_line(),
-        panel.border = element_rect(color = "black", fill = NA),
+        # axis.line = element_line(),
+        # axis.ticks = element_line(),
+        # panel.border = element_rect(color = "black", fill = NA),
         panel.grid.major = element_blank(),
         panel.grid.minor = element_blank()
       )
   }
 
   # Return a list containing the plot and the PCA results data frame
+  colnames(pca_scores)[colnames(pca_scores) == "sample"] <- "Sample"
   return(list(plot = p, df = pca_scores))
 }
 
@@ -277,7 +372,7 @@ pca_sample <- function(lr_scores,
 #' Boxplot Comparison of Ligand-Receptor Interaction Scores Across Groups
 #'
 #' @description
-#' Generates  a boxplot comparing LR (ligand-receptor) interaction scores across sample groups.
+#' Generates  a boxplot comparing ligand-receptor (LR) interaction scores across sample groups
 #' with optional significance testing (t-test or Wilcoxon).
 #'
 #' @param lr_scores Data frame containing LR interaction scores per sample (data frame).
@@ -311,12 +406,17 @@ pca_sample <- function(lr_scores,
 #' # Boxplot of LR Score by group
 #' data(lr_scores_eg)
 #' data(metadata_eg)
+#'
 #' res <- boxplot_lr_group_comparison(
-#'   lr_scores_eg, metadata_eg,
-#'   ligand = "TAC4", receptor = "TACR1",
-#'   sender = "Perivascular", receiver = "Cardiac",
-#'   group_by = "IFN_type", score = "normalized"
-#' )
+#'   lr_scores = lr_scores_eg,
+#'   metadata = metadata_eg,
+#'   ligand = "PSAP",
+#'   receptor = "LRP1",
+#'   sender = "Perivascular",
+#'   receiver = "Fibroblast",
+#'   group_by = "IFN_type",
+#'   score = "normalized"
+#'   )
 #'
 #' print(res$plot)
 #' head(res$df)
@@ -412,6 +512,7 @@ boxplot_lr_group_comparison <- function(lr_scores, metadata,
     )
   }
 
+  colnames(df)[colnames(df) == "sample"] <- "Sample"
   return(list(plot = p, df = df))
 }
 
@@ -420,7 +521,7 @@ boxplot_lr_group_comparison <- function(lr_scores, metadata,
 #' Dotplot of Ligand-Receptor Interaction Scores Against Continuous Group Variable
 #'
 #' @description
-#' Creates a dotplot (scatter plot) of LR interaction scores against a continuous variable
+#' Creates a dotplot (scatter plot) of ligand-receptor (LR) interaction scores against a continuous variable
 #' with optional regression line.
 #'
 #' @param lr_scores Data frame containing LR interaction scores per sample (data frame).
@@ -454,10 +555,14 @@ boxplot_lr_group_comparison <- function(lr_scores, metadata,
 #' # Dotplot of LR Score Against Continuous Group Variable
 #' data(lr_scores_eg)
 #' data(metadata_eg)
+#'
 #' res <- dotplot_lr_continuous_group(
-#'   lr_scores_eg, metadata_eg,
-#'   ligand = "TAC4", receptor = "TACR1",
-#'   sender = "Perivascular", receiver = "Cardiac",
+#'   lr_scores = lr_scores_eg,
+#'   metadata = metadata_eg,
+#'   ligand = "HLA-A",
+#'   receptor = "LILRB2",
+#'   sender = "Lymphoid",
+#'   receiver = "Myeloid",
 #'   group_by = "IFNscore"
 #' )
 #'
@@ -567,7 +672,6 @@ dotplot_lr_continuous_group <- function(lr_scores, metadata,
         color = "black")
   }
 
+  colnames(df)[colnames(df) == "sample"] <- "Sample"
   return(list(plot = p, df = df))
 }
-
-
